#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# VNC monitoring for autots
#
# Copyright (C) 2008 Maximilian Hoegner
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import pygtk
pygtk.require('2.0')

import gtk

import autots
import sys
import os
import threading
import time

# Localisation
_ = autots._

room_windows = {}

def ask(q, w=None):
	d = gtk.MessageDialog(w, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, q)
	r=d.run()
	d.destroy()

	return r

def complain(c, w=None):
	d = gtk.MessageDialog(w, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, c)
	r=d.run()
	d.destroy()

	return r

def update_screenshot(host,image):
		gtk.gdk.threads_enter()
		try:
			image.set_from_file( os.path.join(autots.root,'computer.png') )
		except:
			pass
		gtk.gdk.threads_leave()

		while image.window:
			s = os.system("ping -w3 -c1 "+host+" > /dev/null")

			if s==0:
				cmd = "vncsnapshot -passwd /opt/ltsp/i386/root/.vnc/passwd "+host+" - | convert - -resize 100 xpm:-"
				h = os.popen(cmd,"r")
				xpm = h.readlines()
				h.close()

				xpm = filter(lambda x : x, [(l+'"').split('"')[1] for l in xpm])

				if len(xpm)>0:
					gtk.gdk.threads_enter()
					try:
						pm,tm = gtk.gdk.pixmap_create_from_xpm_d(image.window, None, xpm)
						image.set_from_pixmap(pm, tm)
					except:
						pass
					gtk.gdk.threads_leave()
			else:
				gtk.gdk.threads_enter()
				try:
					image.set_from_file( os.path.join(autots.root,'computer.png') )
				except:
					pass
				gtk.gdk.threads_leave()

			time.sleep(5)

class room_window:
	def __init__(self, roomname):
		self.roomname = roomname

		# window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)

		self.window.set_title(self.roomname)
		self.window.connect("delete-event", self.close)
		self.window.set_default_size(300, 300)

		box = gtk.VBox(False,0)
		self.window.add(box)

		# window content
		table = gtk.Table(2, 2, False)
		box.pack_start(table, True, True, 0)

		hbox=gtk.HBox();
		box.pack_start(hbox, False, False, 0)

		# buttons

		button2 = gtk.Button(stock=gtk.STOCK_CLOSE)
		button2.connect("clicked", self.close)
		hbox.add(button2)

		# room
		self.layout = gtk.Layout(None, None)
		self.layout.set_size(600, 600)
		table.attach(self.layout, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		vScrollbar = gtk.VScrollbar(None)
		vAdjust = self.layout.get_vadjustment()
		vScrollbar.set_adjustment(vAdjust)
		table.attach(vScrollbar, 1, 2, 0, 1, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)

		hScrollbar = gtk.HScrollbar(None)
		hAdjust = self.layout.get_hadjustment()
		hScrollbar.set_adjustment(hAdjust)
		table.attach(hScrollbar, 0, 1, 1, 2, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)

		# computers
		self.computer_buttons = {}
		
		for computer in autots.room_computers(self.roomname):
			self.computer_update_button(computer)

		self.window.show_all()

	def computer_update_button(self,computer):
		if autots.computer_exists(computer) and autots.computer_room(computer)==self.roomname:
			pos = autots.computer_get_position(computer)

			if self.computer_buttons.has_key(computer):
				# update position
				self.layout.move(self.computer_buttons[computer],pos[0],pos[1])
			else:
				# create computer button
				self.computer_buttons[computer] = gtk.Button()

				label = gtk.Label(computer)
				image = gtk.Image()
				threading.Thread(target=update_screenshot,args=(computer,image))
				#thread.start_new_thread(update_screenshot,(computer,image))

				box = gtk.VBox()
				box.add(image)
				box.add(label)

				self.computer_buttons[computer].add(box)

				self.computer_buttons[computer].connect('clicked',self.computer_edit, computer)
				self.computer_buttons[computer].show_all()

				# place button in room
				self.layout.put(self.computer_buttons[computer], pos[0], pos[1])
		else:
			if self.computer_buttons.has_key(computer):
				# delete button
				self.computer_buttons[computer].destroy()
				del self.computer_buttons[computer]

	# Room actions
	
	def close(self, widget, *data):
		self.window.destroy()
		del globals()['room_windows'][self.roomname]

		if len(sys.argv)>1:
			gtk.main_quit()

	# Computer actions

	def computer_edit(self, item, computer):
		if os.environ["DISPLAY"].split(":")[0] != computer:
			c = "vncviewer -passwd /opt/ltsp/i386/root/.vnc/passwd "+computer+" &"
			os.system(c)
		else:
			print "no vnc to own screen"

class network_window:
	"""The network window that lists all rooms in a TreeView"""
	def __init__(self):
		# create ListStore
		self.liststore = gtk.ListStore(str)

		for room in autots.rooms():
			self.liststore.append([room])

		# create TreeView
		self.treeview = gtk.TreeView(self.liststore)
		self.treeview.append_column(gtk.TreeViewColumn(_('room'), gtk.CellRendererText(), text=0))

		self.treeview.connect('row-activated',self.room_doubleclicked)

		# create Buttons
		hbox = gtk.HBox()

		edit = gtk.Button(stock=gtk.STOCK_OPEN)
		edit.connect('clicked', self.edit)
		hbox.add(edit)

		close = gtk.Button(stock=gtk.STOCK_CLOSE)
		close.connect('clicked', self.close)
		hbox.add(close)

		# create window content
		box = gtk.VBox(False,0)

		box.pack_start(gtk.Label(_('Room List')), False)

		sw = gtk.ScrolledWindow()
		box.pack_start(sw, True)
		sw.add(self.treeview)

		box.pack_start(hbox, False)

		# create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_title(_("Network"))
		self.window.connect("destroy", self.close)
		self.window.set_default_size(400, 200)

		self.window.add(box)
		self.window.show_all()

	def room_doubleclicked(self, treeview, path, view_column):
		room = treeview.get_model().get_value(treeview.get_model().get_iter(path), 0)

		if not globals()['room_windows'].has_key(room):
			globals()['room_windows'][room] = room_window(room)

	def edit(self, button):
		try:
			treeselection = self.treeview.get_selection()
			(model, iter) = treeselection.get_selected()
			room = model.get_value(iter, 0)
		except:
			pass
		else:
			if not globals()['room_windows'].has_key(room):
				globals()['room_windows'][room] = room_window(room)

	def close(self, widget, *data):
		gtk.main_quit()

if __name__ == "__main__":
	try:
		autots.load()
	except autots.Error,e:
		dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, str(e)+"\n"+_("You have to run gui.py or autots.py first."))
		dialog.run()
		dialog.destroy()
		sys.exit(1)

	if len(sys.argv)>1:
		try:
			room = sys.argv[1]
			globals()['room_windows'][room] = room_window(room)
		except autots.Error,e:
			dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, str(e))
			dialog.run()
			dialog.destroy()
			sys.exit(1)
	else:
		network_window()

	gtk.gdk.threads_init()
	gtk.main()
